@@ -1,5 +1,5 @@
-# parsetree.py
-# Copyright (C) 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
+# mako/parsetree.py
+# Copyright (C) 2006-2011 the Mako authors and contributors <see AUTHORS file>
 #
 # This module is part of Mako and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -16,14 +16,15 @@ class Node(object):
         self.lineno = lineno
         self.pos = pos
         self.filename = filename
-    
+ 
+    @property
     def exception_kwargs(self):
-        return {'source':self.source, 'lineno':self.lineno, 'pos':self.pos, 'filename':self.filename}
-    exception_kwargs = property(exception_kwargs)
-    
+        return {'source':self.source, 'lineno':self.lineno, 
+                'pos':self.pos, 'filename':self.filename}
+ 
     def get_children(self):
         return []
-        
+ 
     def accept_visitor(self, visitor):
         def traverse(node):
             for n in node.get_children():
@@ -33,27 +34,29 @@ class Node(object):
 
 class TemplateNode(Node):
     """a 'container' node that stores the overall collection of nodes."""
-    
+ 
     def __init__(self, filename):
         super(TemplateNode, self).__init__('', 0, 0, filename)
         self.nodes = []
         self.page_attributes = {}
-        
+ 
     def get_children(self):
         return self.nodes
-        
+ 
     def __repr__(self):
-        return "TemplateNode(%s, %r)" % (util.sorted_dict_repr(self.page_attributes), self.nodes)
-        
+        return "TemplateNode(%s, %r)" % (
+                    util.sorted_dict_repr(self.page_attributes), 
+                    self.nodes)
+ 
 class ControlLine(Node):
     """defines a control line, a line-oriented python line or end tag.
-    
+ 
     e.g.::
 
         % if foo:
             (markup)
         % endif
-    
+ 
     """
 
     def __init__(self, keyword, isend, text, **kwargs):
@@ -75,16 +78,17 @@ class ControlLine(Node):
 
     def undeclared_identifiers(self):
         return self._undeclared_identifiers
-        
+ 
     def is_ternary(self, keyword):
-        """return true if the given keyword is a ternary keyword for this ControlLine"""
-        
+        """return true if the given keyword is a ternary keyword
+        for this ControlLine"""
+ 
         return keyword in {
             'if':set(['else', 'elif']),
             'try':set(['except', 'finally']),
             'for':set(['else'])
         }.get(self.keyword, [])
-        
+ 
     def __repr__(self):
         return "ControlLine(%r, %r, %r, %r)" % (
             self.keyword, 
@@ -95,29 +99,29 @@ class ControlLine(Node):
 
 class Text(Node):
     """defines plain text in the template."""
-    
+ 
     def __init__(self, content, **kwargs):
         super(Text, self).__init__(**kwargs)
         self.content = content
-        
+ 
     def __repr__(self):
         return "Text(%r, %r)" % (self.content, (self.lineno, self.pos))
-        
+ 
 class Code(Node):
     """defines a Python code block, either inline or module level.
-    
+ 
     e.g.::
 
         inline:
         <%
             x = 12
         %>
-    
+ 
         module level:
         <%!
             import logger
         %>
-    
+ 
     """
 
     def __init__(self, text, ismodule, **kwargs):
@@ -138,28 +142,28 @@ class Code(Node):
             self.ismodule, 
             (self.lineno, self.pos)
         )
-        
+ 
 class Comment(Node):
     """defines a comment line.
-    
+ 
     # this is a comment
-    
+ 
     """
-    
+ 
     def __init__(self, text, **kwargs):
         super(Comment, self).__init__(**kwargs)
         self.text = text
 
     def __repr__(self):
         return "Comment(%r, %r)" % (self.text, (self.lineno, self.pos))
-        
+ 
 class Expression(Node):
     """defines an inline expression.
-    
+ 
     ${x+y}
-    
+ 
     """
-    
+ 
     def __init__(self, text, escapes, **kwargs):
         super(Expression, self).__init__(**kwargs)
         self.text = text
@@ -176,7 +180,7 @@ class Expression(Node):
                 self.escapes_code.undeclared_identifiers.difference(
                     set(filters.DEFAULT_ESCAPES.keys())
                 )
-            )
+            ).difference(self.code.declared_identifiers)
 
     def __repr__(self):
         return "Expression(%r, %r, %r)" % (
@@ -184,21 +188,23 @@ class Expression(Node):
             self.escapes_code.args, 
             (self.lineno, self.pos)
         )
-        
+ 
 class _TagMeta(type):
-    """metaclass to allow Tag to produce a subclass according to its keyword"""
-    
+    """metaclass to allow Tag to produce a subclass according to
+    its keyword"""
+ 
     _classmap = {}
-    
+ 
     def __init__(cls, clsname, bases, dict):
         if cls.__keyword__ is not None:
             cls._classmap[cls.__keyword__] = cls
             super(_TagMeta, cls).__init__(clsname, bases, dict)
-            
+ 
     def __call__(cls, keyword, attributes, **kwargs):
         if ":" in keyword:
             ns, defname = keyword.split(':')
-            return type.__call__(CallNamespaceTag, ns, defname, attributes, **kwargs)
+            return type.__call__(CallNamespaceTag, ns, defname, 
+                                        attributes, **kwargs)
 
         try:
             cls = _TagMeta._classmap[keyword]
@@ -211,38 +217,41 @@ class _TagMeta(type):
                 filename=kwargs['filename']
             )
         return type.__call__(cls, keyword, attributes, **kwargs)
-        
+ 
 class Tag(Node):
     """abstract base class for tags.
-    
+ 
     <%sometag/>
-    
+ 
     <%someothertag>
         stuff
     </%someothertag>
-    
+ 
     """
-    
+ 
     __metaclass__ = _TagMeta
     __keyword__ = None
-    
-    def __init__(self, keyword, attributes, expressions, nonexpressions, required, **kwargs):
+ 
+    def __init__(self, keyword, attributes, expressions, 
+                        nonexpressions, required, **kwargs):
         """construct a new Tag instance.
-        
-        this constructor not called directly, and is only called by subclasses.
-        
-        keyword - the tag keyword
-        
-        attributes - raw dictionary of attribute key/value pairs
-        
-        expressions - a set of identifiers that are legal attributes, 
-            which can also contain embedded expressions
-        
-        nonexpressions - a set of identifiers that are legal attributes, 
-            which cannot contain embedded expressions
-        
-        \**kwargs - other arguments passed to the Node superclass (lineno, pos)
-        
+ 
+        this constructor not called directly, and is only called
+        by subclasses.
+ 
+        :param keyword: the tag keyword
+ 
+        :param attributes: raw dictionary of attribute key/value pairs
+ 
+        :param expressions: a set of identifiers that are legal attributes, 
+         which can also contain embedded expressions
+ 
+        :param nonexpressions: a set of identifiers that are legal 
+         attributes, which cannot contain embedded expressions
+ 
+        :param \**kwargs:
+         other arguments passed to the Node superclass (lineno, pos)
+ 
         """
         super(Tag, self).__init__(**kwargs)
         self.keyword = keyword
@@ -251,37 +260,44 @@ class Tag(Node):
         missing = [r for r in required if r not in self.parsed_attributes]
         if len(missing):
             raise exceptions.CompileException(
-                "Missing attribute(s): %s" % ",".join([repr(m) for m in missing]), 
+                "Missing attribute(s): %s" % 
+                    ",".join([repr(m) for m in missing]), 
                 **self.exception_kwargs)
         self.parent = None
         self.nodes = []
-        
+ 
     def is_root(self):
         return self.parent is None
-        
+ 
     def get_children(self):
         return self.nodes
-        
+ 
     def _parse_attributes(self, expressions, nonexpressions):
         undeclared_identifiers = set()
         self.parsed_attributes = {}
         for key in self.attributes:
             if key in expressions:
                 expr = []
-                for x in re.split(r'(\${.+?})', self.attributes[key]):
-                    m = re.match(r'^\${(.+?)}$', x)
+                for x in re.compile(r'(\${.+?})',
+                                    re.S).split(self.attributes[key]):
+                    m = re.compile(r'^\${(.+?)}$', re.S).match(x)
                     if m:
-                        code = ast.PythonCode(m.group(1), **self.exception_kwargs)
-                        undeclared_identifiers = undeclared_identifiers.union(
-                                                        code.undeclared_identifiers
-                                                    )
-                        expr.append("(%s)" % m.group(1))
+                        code = ast.PythonCode(m.group(1).rstrip(),
+                                **self.exception_kwargs)
+                        # we aren't discarding "declared_identifiers" here,
+                        # which we do so that list comprehension-declared 
+                        # variables aren't counted.   As yet can't find a 
+                        # condition that requires it here.
+                        undeclared_identifiers = \
+                            undeclared_identifiers.union(
+                                    code.undeclared_identifiers)
+                        expr.append('(%s)' % m.group(1))
                     else:
                         if x:
                             expr.append(repr(x))
                 self.parsed_attributes[key] = " + ".join(expr) or repr('')
             elif key in nonexpressions:
-                if re.search(r'${.+?}', self.attributes[key]):
+                if re.search(r'\${.+?}', self.attributes[key]):
                     raise exceptions.CompileException(
                             "Attibute '%s' in tag '%s' does not allow embedded "
                             "expressions"  % (key, self.keyword), 
@@ -302,12 +318,12 @@ class Tag(Node):
 
     def __repr__(self):
         return "%s(%r, %s, %r, %r)" % (self.__class__.__name__, 
-                                        self.keyword, 
-                                        util.sorted_dict_repr(self.attributes),
-                                        (self.lineno, self.pos), 
-                                        self.nodes
-                                    )
-        
+                                    self.keyword, 
+                                    util.sorted_dict_repr(self.attributes),
+                                    (self.lineno, self.pos), 
+                                    self.nodes
+                                )
+ 
 class IncludeTag(Tag):
     __keyword__ = 'include'
 
@@ -325,26 +341,34 @@ class IncludeTag(Tag):
         return []
 
     def undeclared_identifiers(self):
-        identifiers = self.page_args.undeclared_identifiers.difference(set(["__DUMMY"]))
-        return identifiers.union(super(IncludeTag, self).undeclared_identifiers())
-    
+        identifiers = self.page_args.undeclared_identifiers.\
+                            difference(set(["__DUMMY"])).\
+                            difference(self.page_args.declared_identifiers)
+        return identifiers.union(super(IncludeTag, self).
+                                    undeclared_identifiers())
+ 
 class NamespaceTag(Tag):
     __keyword__ = 'namespace'
 
     def __init__(self, keyword, attributes, **kwargs):
         super(NamespaceTag, self).__init__(
                                         keyword, attributes, 
-                                        (), 
+                                        ('file',), 
                                         ('name','inheritable',
-                                        'file','import','module'), 
+                                        'import','module'), 
                                         (), **kwargs)
-                                        
+ 
         self.name = attributes.get('name', '__anon_%s' % hex(abs(id(self))))
         if not 'name' in attributes and not 'import' in attributes:
             raise exceptions.CompileException(
-                                "'name' and/or 'import' attributes are required "
-                                "for <%namespace>", 
-                                **self.exception_kwargs)
+                "'name' and/or 'import' attributes are required "
+                "for <%namespace>",
+                **self.exception_kwargs)
+        if 'file' in attributes and 'module' in attributes:
+            raise exceptions.CompileException(
+                "<%namespace> may only have one of 'file' or 'module'",
+                **self.exception_kwargs
+            )
 
     def declared_identifiers(self):
         return []
@@ -360,7 +384,7 @@ class TextTag(Tag):
         self.filter_args = ast.ArgumentList(
                                     attributes.get('filter', ''), 
                                     **self.exception_kwargs)
-        
+ 
 class DefTag(Tag):
     __keyword__ = 'def'
 
@@ -378,25 +402,88 @@ class DefTag(Tag):
             raise exceptions.CompileException(
                                 "Missing parenthesis in %def", 
                                 **self.exception_kwargs)
-        self.function_decl = ast.FunctionDecl("def " + name + ":pass", **self.exception_kwargs)
+        self.function_decl = ast.FunctionDecl("def " + name + ":pass", 
+                                                    **self.exception_kwargs)
         self.name = self.function_decl.funcname
         self.decorator = attributes.get('decorator', '')
         self.filter_args = ast.ArgumentList(
                                 attributes.get('filter', ''), 
                                 **self.exception_kwargs)
 
+    is_anonymous = False
+    is_block = False
+
+    @property
+    def funcname(self):
+        return self.function_decl.funcname
+
+    def get_argument_expressions(self, **kw):
+        return self.function_decl.get_argument_expressions(**kw)
+
     def declared_identifiers(self):
         return self.function_decl.argnames
 
     def undeclared_identifiers(self):
         res = []
         for c in self.function_decl.defaults:
-            res += list(ast.PythonCode(c, **self.exception_kwargs).undeclared_identifiers)
+            res += list(ast.PythonCode(c, **self.exception_kwargs).
+                                    undeclared_identifiers)
         return res + list(self.filter_args.\
                             undeclared_identifiers.\
                             difference(filters.DEFAULT_ESCAPES.keys())
                         )
 
+class BlockTag(Tag):
+    __keyword__ = 'block'
+
+    def __init__(self, keyword, attributes, **kwargs):
+        super(BlockTag, self).__init__(
+                keyword, 
+                attributes, 
+                ('buffered', 'cached', 'cache_key', 'cache_timeout', 
+                    'cache_type', 'cache_dir', 'cache_url', 'args'), 
+                ('name','filter', 'decorator'), 
+                (), 
+                **kwargs)
+        name = attributes.get('name')
+        if name and not re.match(r'^[\w_]+$',name):
+            raise exceptions.CompileException(
+                                "%block may not specify an argument signature", 
+                                **self.exception_kwargs)
+        if not name and attributes.get('args', None):
+            raise exceptions.CompileException(
+                                "Only named %blocks may specify args",
+                                **self.exception_kwargs
+                                )
+        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), 
+                                            **self.exception_kwargs)
+
+        self.name = name
+        self.decorator = attributes.get('decorator', '')
+        self.filter_args = ast.ArgumentList(
+                                attributes.get('filter', ''), 
+                                **self.exception_kwargs)
+
+
+    is_block = True
+
+    @property
+    def is_anonymous(self):
+        return self.name is None
+
+    @property
+    def funcname(self):
+        return self.name or "__M_anon_%d" % (self.lineno, )
+
+    def get_argument_expressions(self, **kw):
+        return self.body_decl.get_argument_expressions(**kw)
+
+    def declared_identifiers(self):
+        return self.body_decl.argnames
+
+    def undeclared_identifiers(self):
+        return []
+
 class CallTag(Tag):
     __keyword__ = 'call'
 
@@ -405,13 +492,15 @@ class CallTag(Tag):
                                     ('args'), ('expr',), ('expr',), **kwargs)
         self.expression = attributes['expr']
         self.code = ast.PythonCode(self.expression, **self.exception_kwargs)
-        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), **self.exception_kwargs)
+        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), 
+                                            **self.exception_kwargs)
 
     def declared_identifiers(self):
         return self.code.declared_identifiers.union(self.body_decl.argnames)
 
     def undeclared_identifiers(self):
-        return self.code.undeclared_identifiers
+        return self.code.undeclared_identifiers.\
+                    difference(self.code.declared_identifiers)
 
 class CallNamespaceTag(Tag):
 
@@ -423,7 +512,7 @@ class CallNamespaceTag(Tag):
                     (), 
                     (), 
                     **kwargs)
-                    
+ 
         self.expression = "%s.%s(%s)" % (
                                 namespace, 
                                 defname, 
@@ -440,7 +529,8 @@ class CallNamespaceTag(Tag):
         return self.code.declared_identifiers.union(self.body_decl.argnames)
 
     def undeclared_identifiers(self):
-        return self.code.undeclared_identifiers
+        return self.code.undeclared_identifiers.\
+                    difference(self.code.declared_identifiers)
 
 class InheritTag(Tag):
     __keyword__ = 'inherit'
@@ -463,12 +553,13 @@ class PageTag(Tag):
                 (), 
                 (), 
                 **kwargs)
-        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), **self.exception_kwargs)
+        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), 
+                                            **self.exception_kwargs)
         self.filter_args = ast.ArgumentList(
                                 attributes.get('expression_filter', ''),
                                 **self.exception_kwargs)
 
     def declared_identifiers(self):
         return self.body_decl.argnames
-        
-    
+ 
+ 
